@@ -42,6 +42,7 @@
 #include "udiskslvm2daemonutil.h"
 #include "udiskslvm2dbusutil.h"
 #include "udisks-lvm2-generated.h"
+#include "jobhelpers.h"
 
 /**
  * SECTION:udiskslinuxvolumegroupobject
@@ -66,7 +67,7 @@ struct _UDisksLinuxVolumeGroupObject
   gchar *name;
 
   GHashTable *logical_volumes;
-  GPid poll_pid;
+  guint32 poll_epoch;
   guint poll_timeout_id;
   gboolean poll_requested;
 
@@ -92,6 +93,11 @@ static void etctabs_changed (UDisksFstabMonitor *monitor,
                              UDisksFstabEntry   *entry,
                              gpointer            user_data);
 
+typedef struct {
+  BDLVMVGdata *vg_info;
+  GSList *vg_pvs;
+} VGUpdateData;
+
 static void
 udisks_linux_volume_group_object_finalize (GObject *_object)
 {
@@ -171,6 +177,9 @@ udisks_linux_volume_group_object_set_property (GObject      *__object,
 static void
 udisks_linux_volume_group_object_init (UDisksLinuxVolumeGroupObject *object)
 {
+  object->poll_epoch = 0;
+  object->poll_timeout_id = 0;
+  object->poll_requested = FALSE;
 }
 
 static void
@@ -384,20 +393,18 @@ update_progress_for_device (UDisksDaemon *daemon,
 static void
 update_operations (UDisksDaemon *daemon,
                    const gchar  *lv_name,
-                   GVariant     *lv_info,
+                   BDLVMLVdata  *lv_info,
                    gboolean     *needs_polling_ret)
 {
-  const gchar *move_pv;
-  guint64 copy_percent;
-
-  if (lv_is_pvmove_volume (lv_name)
-      && g_variant_lookup (lv_info, "move_pv", "&s", &move_pv)
-      && g_variant_lookup (lv_info, "copy_percent", "t", &copy_percent))
+  if (lv_is_pvmove_volume (lv_name))
     {
-      update_progress_for_device (daemon,
-                                  "lvm-vg-empty-device",
-                                  move_pv,
-                                  copy_percent/100000000.0);
+      if (lv_info->move_pv && lv_info->copy_percent)
+        {
+          update_progress_for_device (daemon,
+                                      "lvm-vg-empty-device",
+                                      lv_info->move_pv,
+                                      lv_info->copy_percent/100.0);
+        }
       *needs_polling_ret = TRUE;
     }
 }
@@ -422,6 +429,21 @@ block_object_update_lvm_iface (UDisksLinuxBlockObject *object,
   udisks_block_lvm2_set_logical_volume (iface_block_lvm2, lv_obj_path);
 }
 
+static void
+lv_object_update_block_path (UDisksLinuxBlockObject       *block_object,
+                             UDisksLinuxLogicalVolumeObject *lv_object)
+{
+  UDisksLogicalVolume *lv = NULL;
+  const char *block_objpath = NULL;
+
+  lv = udisks_object_peek_logical_volume (UDISKS_OBJECT (lv_object));
+  if (lv)
+    {
+      block_objpath = g_dbus_object_get_object_path (G_DBUS_OBJECT (block_object));
+      udisks_logical_volume_set_block_device (UDISKS_LOGICAL_VOLUME (lv), block_objpath);
+    }
+}
+
 static void
 update_block (UDisksLinuxBlockObject       *block_object,
               UDisksLinuxVolumeGroupObject *group_object,
@@ -429,7 +451,7 @@ update_block (UDisksLinuxBlockObject       *block_object,
               GHashTable                   *new_pvs)
 {
   UDisksBlock *block;
-  GVariant *pv_info;
+  BDLVMPVdata *pv_info;
 
   block = udisks_object_peek_block (UDISKS_OBJECT (block_object));
   if (block == NULL)
@@ -452,6 +474,7 @@ update_block (UDisksLinuxBlockObject       *block_object,
             && (lv_object = g_hash_table_lookup (new_lvs, block_lv_name)))
           {
             block_object_update_lvm_iface (block_object, g_dbus_object_get_object_path (G_DBUS_OBJECT (lv_object)));
+            lv_object_update_block_path (block_object, lv_object);
           }
 
         g_object_unref(device);
@@ -485,75 +508,120 @@ update_block (UDisksLinuxBlockObject       *block_object,
     }
 }
 
+/**
+ * cmp_int_lv_name: (skip)
+ *
+ * Compare name of an internal LV (possibly enclosed in square brackets) with a
+ * given other name.
+ */
+static gboolean
+cmp_int_lv_name (const gchar *int_lv_name, const gchar *lv_name)
+{
+  const gchar *c = NULL;
+  if (!int_lv_name || !lv_name)
+    return FALSE;
+
+  if (*int_lv_name == '[')
+    int_lv_name++;
+
+  for (c=int_lv_name; *c != '\0' && *c != ']'; c++)
+    if (*c != lv_name[c - int_lv_name])
+      return FALSE;
+
+  if (*c == ']')
+    c++;
+  if (*c == '\0')
+    return TRUE;
+
+  return FALSE;
+}
+
 static void
-update_with_variant (GPid pid,
-                     GVariant *info,
-                     GError *error,
-                     gpointer user_data)
+update_vg (GObject      *source_obj,
+           GAsyncResult *result,
+           gpointer      user_data)
 {
-  UDisksLinuxVolumeGroupObject *object = user_data;
   UDisksDaemon *daemon;
   GDBusObjectManagerServer *manager;
-  GVariantIter *iter;
   GHashTableIter volume_iter;
   gpointer key, value;
   GHashTable *new_lvs;
   GHashTable *new_pvs;
   GList *objects, *l;
   gboolean needs_polling = FALSE;
+  GError *error = NULL;
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
-  manager = udisks_daemon_get_object_manager (daemon);
+  UDisksLinuxVolumeGroupObject *object = UDISKS_LINUX_VOLUME_GROUP_OBJECT (source_obj);
+  GTask *task = G_TASK (result);
+  VGUpdateData *data = user_data;
+  BDLVMLVdata **lvs = g_task_propagate_pointer (task, &error);
+  BDLVMVGdata *vg_info = data->vg_info;
+  GSList *vg_pvs = data->vg_pvs;
 
-  if (error)
+  /* free the data container (but not 'vg_info' and 'vg_pvs') */
+  g_free (data);
+
+  if (!lvs)
     {
-      udisks_warning ("Failed to update LVM volume group %s: %s",
-                      udisks_linux_volume_group_object_get_name (object),
-                      error->message);
+      if (error)
+        {
+          udisks_warning ("Failed to update LVM volume group %s: %s",
+                          udisks_linux_volume_group_object_get_name (object),
+                          error->message);
+        }
+      else
+        {
+          /* this should never happen */
+          udisks_warning ("Failed to update LVM volume group %s: no error reported",
+                          udisks_linux_volume_group_object_get_name (object));
+        }
+      g_slist_free_full (vg_pvs, (GDestroyNotify) bd_lvm_pvdata_free);
+      bd_lvm_vgdata_free (vg_info);
       g_object_unref (object);
       return;
     }
 
-  udisks_linux_volume_group_update (UDISKS_LINUX_VOLUME_GROUP (object->iface_volume_group), info, &needs_polling);
+  daemon = udisks_linux_volume_group_object_get_daemon (object);
+  manager = udisks_daemon_get_object_manager (daemon);
+
+  udisks_linux_volume_group_update (UDISKS_LINUX_VOLUME_GROUP (object->iface_volume_group), vg_info, &needs_polling);
 
   if (!g_dbus_object_manager_server_is_exported (manager, G_DBUS_OBJECT_SKELETON (object)))
     g_dbus_object_manager_server_export_uniquely (manager, G_DBUS_OBJECT_SKELETON (object));
 
   new_lvs = g_hash_table_new (g_str_hash, g_str_equal);
 
-  if (g_variant_lookup (info, "lvs", "aa{sv}", &iter))
+  for (BDLVMLVdata **lvs_p=lvs; *lvs_p; lvs_p++)
     {
-      GVariant *lv_info = NULL;
-      while (g_variant_iter_loop (iter, "@a{sv}", &lv_info))
+      UDisksLinuxLogicalVolumeObject *volume;
+      BDLVMLVdata *lv_info = *lvs_p;
+      const gchar *lv_name = lv_info->lv_name;
+      BDLVMLVdata *meta_lv_info = NULL;
+
+      update_operations (daemon, lv_name, lv_info, &needs_polling);
+
+      if (udisks_daemon_util_lvm2_name_is_reserved (lv_name))
+        continue;
+
+      if (lv_info->metadata_lv && *(lv_info->metadata_lv) != '\0')
+        /* this is not cheap to do, but not many LVs have a metadata LV */
+        for (BDLVMLVdata **lvs_p2=lvs; !meta_lv_info && *lvs_p2; lvs_p2++)
+          if (cmp_int_lv_name ((*lvs_p2)->lv_name, lv_info->metadata_lv))
+            meta_lv_info = *lvs_p2;
+
+      volume = g_hash_table_lookup (object->logical_volumes, lv_name);
+      if (volume == NULL)
         {
-          const gchar *name;
-          UDisksLinuxLogicalVolumeObject *volume;
-
-          g_variant_lookup (lv_info, "name", "&s", &name);
-
-          update_operations (daemon, name, lv_info, &needs_polling);
-
-          if (lv_is_pvmove_volume (name))
-            needs_polling = TRUE;
-
-          if (udisks_daemon_util_lvm2_name_is_reserved (name))
-            continue;
-
-          volume = g_hash_table_lookup (object->logical_volumes, name);
-          if (volume == NULL)
-            {
-              volume = udisks_linux_logical_volume_object_new (daemon, object, name);
-              udisks_linux_logical_volume_object_update (volume, lv_info, &needs_polling);
-              udisks_linux_logical_volume_object_update_etctabs (volume);
-              g_dbus_object_manager_server_export_uniquely (manager, G_DBUS_OBJECT_SKELETON (volume));
-              g_hash_table_insert (object->logical_volumes, g_strdup (name), g_object_ref (volume));
-            }
-          else
-            udisks_linux_logical_volume_object_update (volume, lv_info, &needs_polling);
-
-          g_hash_table_insert (new_lvs, (gchar *)name, volume);
+          volume = udisks_linux_logical_volume_object_new (daemon, object, lv_name);
+          udisks_linux_logical_volume_object_update (volume, lv_info, meta_lv_info, &needs_polling);
+          udisks_linux_logical_volume_object_update_etctabs (volume);
+          g_dbus_object_manager_server_export_uniquely (manager, G_DBUS_OBJECT_SKELETON (volume));
+          g_hash_table_insert (object->logical_volumes, g_strdup (lv_name), g_object_ref (volume));
         }
-      g_variant_iter_free (iter);
+      else
+        udisks_linux_logical_volume_object_update (volume, lv_info, meta_lv_info, &needs_polling);
+
+      g_hash_table_insert (new_lvs, (gchar *)lv_name, volume);
     }
 
   g_hash_table_iter_init (&volume_iter, object->logical_volumes);
@@ -576,19 +644,13 @@ update_with_variant (GPid pid,
                                          needs_polling);
 
   /* Update block objects. */
-
-  new_pvs = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, (GDestroyNotify)g_variant_unref);
-  if (g_variant_lookup (info, "pvs", "aa{sv}", &iter))
+  new_pvs = g_hash_table_new (g_str_hash, g_str_equal);
+  for (GSList *vg_pvs_p=vg_pvs; vg_pvs_p; vg_pvs_p=vg_pvs_p->next)
     {
-      const gchar *name;
-      GVariant *pv_info;
-      while (g_variant_iter_next (iter, "@a{sv}", &pv_info))
-        {
-          if (g_variant_lookup (pv_info, "device", "&s", &name))
-            g_hash_table_insert (new_pvs, (gchar *)name, pv_info);
-          else
-            g_variant_unref (pv_info);
-        }
+      BDLVMPVdata *pv_info = vg_pvs_p->data;
+      gchar *pv_name = pv_info->pv_name;
+      if (pv_name)
+        g_hash_table_insert (new_pvs, pv_name, pv_info);
     }
 
   objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (manager));
@@ -602,73 +664,94 @@ update_with_variant (GPid pid,
   g_hash_table_destroy (new_lvs);
   g_hash_table_destroy (new_pvs);
 
+  g_slist_free_full (vg_pvs, (GDestroyNotify) bd_lvm_pvdata_free);
+  bd_lvm_vgdata_free (vg_info);
+  lv_list_free (lvs);
+
   g_object_unref (object);
 }
 
 void
-udisks_linux_volume_group_object_update (UDisksLinuxVolumeGroupObject *object)
+udisks_linux_volume_group_object_update (UDisksLinuxVolumeGroupObject *object, BDLVMVGdata *vg_info, GSList *pvs)
 {
-  UDisksDaemon *daemon = udisks_linux_volume_group_object_get_daemon (object);
+  VGUpdateData *data = g_new0 (VGUpdateData, 1);
+  gchar *vg_name = g_strdup (vg_info->name);
+  GTask *task = NULL;
 
-  const gchar *args[] = { NULL, "-b", "show", object->name, NULL };
-  if (udisks_daemon_get_uninstalled (daemon))
-    args[0] = BUILD_DIR "modules/lvm2/udisks-lvm";
-  else
-    args[0] = LVM_HELPER_DIR "udisks-lvm";
+  data->vg_info = vg_info;
+  data->vg_pvs = pvs;
 
-  udisks_daemon_util_lvm2_spawn_for_variant (args, G_VARIANT_TYPE("a{sv}"),
-                                             update_with_variant, g_object_ref (object));
+  /* the callback (update_vg) is called in the default main loop (context) */
+  task = g_task_new (g_object_ref (object), NULL /* cancellable */, update_vg, data /* callback_data */);
+  g_task_set_task_data (task, vg_name, g_free);
+
+  /* holds a reference to 'task' until it is finished */
+  g_task_run_in_thread (task, (GTaskThreadFunc) lvs_task_func);
+
+  g_object_unref (task);
 }
 
 static void
-poll_with_variant (GPid pid,
-                   GVariant *info,
-                   GError *error,
-                   gpointer user_data)
+poll_vg_update (GObject      *source_obj,
+                GAsyncResult *result,
+                gpointer      user_data)
 {
-  UDisksLinuxVolumeGroupObject *object = user_data;
   UDisksDaemon *daemon;
-  GVariantIter *iter;
   gboolean needs_polling;
+  GError *error = NULL;
+  UDisksLinuxVolumeGroupObject *object = UDISKS_LINUX_VOLUME_GROUP_OBJECT (source_obj);
+  GTask *task = G_TASK (result);
+  guint32 epoch_started = GPOINTER_TO_UINT (user_data);
+  BDLVMLVdata **lvs = g_task_propagate_pointer (task, &error);
 
-  if (pid != object->poll_pid)
+  if (epoch_started != object->poll_epoch)
     {
+      /* epoch has changed -> another poll update is on the way */
+      lv_list_free (lvs);
       g_object_unref (object);
       return;
     }
 
-  object->poll_pid = 0;
-
-  if (error)
+  if (!lvs)
     {
-      udisks_warning ("Failed to poll LVM volume group %s: %s",
-                      udisks_linux_volume_group_object_get_name (object),
-                      error->message);
+      if (error)
+        udisks_warning ("Failed to poll LVM volume group %s: %s",
+                        udisks_linux_volume_group_object_get_name (object),
+                        error->message);
+      else
+        /* this should never happen */
+        udisks_warning ("Failed to poll LVM volume group %s: no error reported",
+                        udisks_linux_volume_group_object_get_name (object));
+
       g_object_unref (object);
       return;
     }
 
   daemon = udisks_linux_volume_group_object_get_daemon (object);
 
-  udisks_linux_volume_group_update (UDISKS_LINUX_VOLUME_GROUP (object->iface_volume_group), info, &needs_polling);
+  /* XXX: we used to do this, but it seems to be pointless (how could a VG change without emitting a uevent on the PVs?) */
+  /* udisks_linux_volume_group_update (UDISKS_LINUX_VOLUME_GROUP (object->iface_volume_group), info, &needs_polling); */
 
-  if (g_variant_lookup (info, "lvs", "aa{sv}", &iter))
+  for (BDLVMLVdata **lvs_p=lvs; *lvs_p; lvs_p++)
     {
-      GVariant *lv_info = NULL;
-      while (g_variant_iter_loop (iter, "@a{sv}", &lv_info))
-        {
-          const gchar *name;
-          UDisksLinuxLogicalVolumeObject *volume;
+      UDisksLinuxLogicalVolumeObject *volume;
+      BDLVMLVdata *lv_info = *lvs_p;
+      BDLVMLVdata *meta_lv_info = NULL;
+      const gchar *lv_name = lv_info->lv_name;
 
-          g_variant_lookup (lv_info, "name", "&s", &name);
-          update_operations (daemon, name, lv_info, &needs_polling);
-          volume = g_hash_table_lookup (object->logical_volumes, name);
-          if (volume)
-            udisks_linux_logical_volume_object_update (volume, lv_info, &needs_polling);
-        }
-      g_variant_iter_free (iter);
+      if (lv_info->metadata_lv && *(lv_info->metadata_lv) != '\0')
+        /* this is not cheap to do, but not many LVs have a metadata LV */
+        for (BDLVMLVdata **lvs_np=lvs; !meta_lv_info && *lvs_np; lvs_np++)
+          if (cmp_int_lv_name ((*lvs_np)->lv_name, lv_info->metadata_lv))
+            meta_lv_info = *lvs_np;
+
+      update_operations (daemon, lv_name, lv_info, &needs_polling);
+      volume = g_hash_table_lookup (object->logical_volumes, lv_name);
+      if (volume)
+        udisks_linux_logical_volume_object_update (volume, lv_info, meta_lv_info, &needs_polling);
     }
 
+  lv_list_free (lvs);
   g_object_unref (object);
 }
 
@@ -707,15 +790,23 @@ poll_timeout (gpointer user_data)
 static void
 poll_now (UDisksLinuxVolumeGroupObject *object)
 {
-  const gchar *args[] = { LVM_HELPER_DIR "udisks-lvm", "-b", "show", object->name, NULL };
+  gchar *vg_name = g_strdup (udisks_linux_volume_group_object_get_name (object));
+  GTask *task = NULL;
 
   object->poll_timeout_id = g_timeout_add (5000, poll_timeout, g_object_ref (object));
 
-  if (object->poll_pid)
-    kill (object->poll_pid, SIGINT);
+  /* starting a new poll -> increment the epoch */
+  object->poll_epoch++;
 
-  object->poll_pid = udisks_daemon_util_lvm2_spawn_for_variant (args, G_VARIANT_TYPE("a{sv}"),
-                                                                poll_with_variant, g_object_ref (object));
+  /* the callback (poll_vg_update) is called in the default main loop (context) */
+  task = g_task_new (g_object_ref (object), NULL /* cancellable */,
+                     poll_vg_update, GUINT_TO_POINTER (object->poll_epoch) /* callback_data */);
+  g_task_set_task_data (task, vg_name, g_free);
+
+  /* holds a reference to 'task' until it is finished */
+  g_task_run_in_thread (task, (GTaskThreadFunc) lvs_task_func);
+
+  g_object_unref (task);
 }
 
 void
