@@ -13,6 +13,7 @@ from gi.repository import GLib
 
 import safe_dbus
 import udiskstestcase
+from udiskstestcase import unstable_test
 
 
 class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
@@ -24,10 +25,8 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
     username = 'udisks_test_user'
 
-    def _clean_format(self, disk):
-        d = dbus.Dictionary(signature='sv')
-        d['erase'] = True
-        disk.Format('empty', d, dbus_interface=self.iface_prefix + '.Block')
+    def _clean_format(self, disk_path):
+        self.run_command('wipefs -a %s' % disk_path)
 
     def _unmount(self, disk_path):
         self.run_command('umount %s' % disk_path)
@@ -71,7 +70,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # test dbus properties
         usage = self.get_property(disk, '.Block', 'IdUsage')
@@ -102,7 +101,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
         d = dbus.Dictionary(signature='sv')
         d['label'] = label
         disk.Format(self._fs_name, d, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # test dbus properties
         dbus_label = self.get_property(disk, '.Block', 'IdLabel')
@@ -124,7 +123,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
         # create filesystem with label
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # change the label
         label = 'AAAA' if self._fs_name == 'vfat' else 'aaaa'  # XXX udisks changes vfat labels to uppercase
@@ -163,7 +162,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # not mounted
         mounts = self.get_property(disk, '.Filesystem', 'MountPoints')
@@ -187,7 +186,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # not mounted
         mounts = self.get_property(disk, '.Filesystem', 'MountPoints')
@@ -200,18 +199,18 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
         mnt_path = disk.Mount(d, dbus_interface=self.iface_prefix + '.Filesystem')
         self.addCleanup(self._unmount, self.vdevs[0])
 
-        # dbus mountpoint
-        dbus_mounts = self.get_property(disk, '.Filesystem', 'MountPoints')
-        dbus_mounts.assertLen(1)  # just one mountpoint
-        dbus_mnt = self.ay_to_str(dbus_mounts.value[0])  # mountpoints are arrays of bytes
-        self.assertEqual(dbus_mnt, mnt_path)
-
         # system mountpoint
         self.assertTrue(os.path.ismount(mnt_path))
         _ret, out = self.run_command('mount | grep %s' % self.vdevs[0])
         self.assertIn(mnt_path, out)
         self.assertIn('ro', out)
 
+        # dbus mountpoint
+        dbus_mounts = self.get_property(disk, '.Filesystem', 'MountPoints')
+        dbus_mounts.assertLen(1)  # just one mountpoint
+        dbus_mnt = self.ay_to_str(dbus_mounts.value[0])  # mountpoints are arrays of bytes
+        self.assertEqual(dbus_mnt, mnt_path)
+
         # umount
         disk.Unmount(self.no_options, dbus_interface=self.iface_prefix + '.Filesystem')
         self.assertFalse(os.path.ismount(mnt_path))
@@ -232,7 +231,7 @@ class UdisksFSTestCase(udiskstestcase.UdisksTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # create a tempdir
         tmp = tempfile.mkdtemp()
@@ -312,7 +311,7 @@ class Ext4TestCase(Ext2TestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # create user for our test
         self.addCleanup(self._remove_user, self.username)
@@ -547,7 +546,7 @@ class VFATTestCase(UdisksFSTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # create user for our test
         self.addCleanup(self._remove_user, self.username)
@@ -585,7 +584,7 @@ class VFATTestCase(UdisksFSTestCase):
 
         # create filesystem
         disk.Format(self._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         # create user for our test
         self.addCleanup(self._remove_user, self.username)
@@ -626,6 +625,9 @@ class VFATTestCase(UdisksFSTestCase):
         self.assertTrue(os.path.ismount(mnt_path))
         self._unmount(mnt_path)
 
+    @unstable_test
+    def test_repair_resize_check(self):
+        super(VFATTestCase, self).test_repair_resize_check()
 
 class NTFSTestCase(UdisksFSTestCase):
     _fs_name = 'ntfs'
@@ -634,6 +636,9 @@ class NTFSTestCase(UdisksFSTestCase):
     _can_relabel = True and UdisksFSTestCase.command_exists('ntfslabel')
     _can_mount = True
 
+    @unstable_test
+    def test_repair_resize_check(self):
+        super(NTFSTestCase, self).test_repair_resize_check()
 
 class BTRFSTestCase(UdisksFSTestCase):
     _fs_name = 'btrfs'
@@ -729,7 +734,7 @@ class FailsystemTestCase(UdisksFSTestCase):
 
         # create udf filesystem without label and try to set it later
         disk.Format(fs._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
 
         msg = 'org.freedesktop.UDisks2.Error.NotSupported: Don\'t know how to '\
               'change label on device of type filesystem:%s' % fs._fs_name
@@ -748,7 +753,7 @@ class FailsystemTestCase(UdisksFSTestCase):
         self.assertIsNotNone(disk)
 
         disk.Format(fs._fs_name, self.no_options, dbus_interface=self.iface_prefix + '.Block')
-        self.addCleanup(self._clean_format, disk)
+        self.addCleanup(self._clean_format, self.vdevs[0])
         self.addCleanup(self._unmount, self.vdevs[0])  # paranoid cleanup
 
         # wrong fstype
