@@ -1,8 +1,11 @@
+from __future__ import print_function
+
 import unittest
 import dbus
 import subprocess
 import os
 import time
+import sys
 from datetime import datetime
 from systemd import journal
 from monotonic import monotonic
@@ -60,6 +63,28 @@ def skip_on(skip_on_distros, skip_on_version="", reason=""):
 
     return decorator
 
+def unstable_test(test):
+    """Decorator for unstable tests
+
+    Failures of tests decorated with this decorator are silently ignored unless
+    the ``UNSTABLE_TESTS_FATAL`` environment variable is defined.
+    """
+
+    def decorated_test(*args):
+        try:
+            test(*args)
+        except unittest.SkipTest:
+            # make sure skipped tests are just skipped as usual
+            raise
+        except:
+            # and swallow everything else, just report a failure of an unstable
+            # test, unless told otherwise
+            if "UNSTABLE_TESTS_FATAL" in os.environ:
+                raise
+            print("unstable-fail...", end="", file=sys.stderr)
+
+    return decorated_test
+
 
 class DBusProperty(object):
 
@@ -108,6 +133,22 @@ class DBusProperty(object):
             else:
                 raise AssertionError('%s != %s' % (self._value, value))
 
+    def assertAlmostEqual(self, value, delta, timeout=TIMEOUT, getter=None):
+        if getter is not None:
+            check_fn = lambda x: abs(getter(x) - value) <= delta
+        else:
+            check_fn = lambda x: abs(x - value) <= delta
+        ret = self._check(timeout, check_fn)
+
+        if not ret:
+            if getter is not None:
+                raise AssertionError('%s is not almost equal to %s (delta = %s)' % (getter(self._value),
+                                                                                    value, delta))
+            else:
+                raise AssertionError('%s is not almost equal to %s (delta = %s)' % (self._value,
+                                                                                    value, delta))
+
+
     def assertGreater(self, value, timeout=TIMEOUT):
         check_fn = lambda x: x > value
         ret = self._check(timeout, check_fn)
